#include "MSP430x44x.h"

LCD_DIV_32		.equ	000h
LCD_DIV_64		.equ	008h
LCD_DIV_128		.equ	010h
LCD_DIV_256		.equ	018h





;************************************************************************
; Reset : Initialize processor
;************************************************************************


.pseg code,abs=01100h

;************************************************************************
; Reset : Initialize processor
;************************************************************************
RESET   mov #00300h,SP
  mov #(WDTPW+WDTHOLD),&WDTCTL  ; Stop Watchdog Timer
  bis.b #XCAP18PF,&FLL_CTL0       ;set load capacitance for xtal
  

	; Initialize UART
	call #Init_UART

	; Setup Basic Timer for LCD operation
	mov.b #(LCD_DIV_64+023h),&BTCTL

	; Turn on Basic Timer interrupts
	bis.b #BTIE,&IE2

	; Setup module functions
	mov.b #032h,&P1SEL
	mov.b #000h,&P2SEL
	mov.b #000h,&P3SEL
	mov.b #0C3h,&P4SEL		; LCD/UART1
	mov.b #0FFh,&P5SEL		; LCD

	; Setup LCD
	mov.b #(0E0h|018h|004h|001h),&LCDCTL
;	mov.b #(LCDLASTSEG35|LCDMODE_4MUX|LCDSHOW|LCDMODULEON),&LCDCTL
	; Setup LCD data
	mov.b #000h,&LCDM1	; S1:S0	
	mov.b #080h,&LCDM2	; S3:S2

	; 78deg F
	mov.b #072h,&LCDM3	; S5:S4
	mov.b #033h,&LCDM4	; S7:S6
	mov.b #0F7h,&LCDM5	; S9:S8
	mov.b #007h,&LCDM6	; S11:S10

	mov.b #0E2h,&LCDM7	; S13:S12
	mov.b #044h,&LCDM8	; S15:S14

	mov.b #062h,&LCDM9	; S17:S16
	mov.b #044h,&LCDM10	; S19:S18

	mov.b #062h,&LCDM11	; S21:S20
	mov.b #044h,&LCDM12	; S23:S22

	mov.b #0E2h,&LCDM13	; S25:S24
	mov.b #064h,&LCDM14	; S27:S26
;	mov.b #001h,&LCDM13	; S25:S24	; Alternate 'A'
;	mov.b #06Ch,&LCDM14	; S27:S26

	mov.b #0C2h,&LCDM15	; S29:S28
	mov.b #064h,&LCDM16	; S31:S30

	mov.b #062h,&LCDM17	; S33:S32
	mov.b #0F4h,&LCDM18	; S35:S34

	; Setup LED outputs
	bis.b #0A0h,&P3DIR

	; Clear LEDs
	bic.b #0A0h,&P3OUT
	; Setup Blinky
	mov.b #033h,&P1DIR

	; Delay
	mov #08000h,R4
DelayLoopTop
	dec R4
	jnz DelayLoopTop

	; Setup P2.1, P2.2, and P2.3 as inputs
;	mov.b #031h,&P2DIR

	; Initialize accessory state
	mov #Display_0,R4

	eint

;******************************************************************************
; Mainloop
;******************************************************************************
LoopTop
	xor.b #020h,&P3OUT    ; Toggle LED
	bic.b #040h,&P1OUT	  ; Clear P1.6

	; Go to sleep
	bis #(CPUOFF+SCG1+SCG0),SR	
	bis.b #040h,&P1OUT	  ; Set P1.6

LoopDemoStuff_Next

	; Demo the symbology
	call R4

	; Check the switch input
	bit.b #040h,&P1IN
	jnz SwitchOff
	
SwitchOn
	; Turn on second LED
	bic.b #080h,&P3OUT
	jmp NextLoop

SwitchOff	
	; Turn off second LED
	bis.b #080h,&P3OUT
	; Intentional fall-through...

NextLoop
	jmp LoopTop

Display_0
	mov #Display_1,R4

	mov.b #081h,&LCDM2	; S3:S2

	ret

Display_1
	mov #Display_2,R4
	
	mov.b #081h,&LCDM2	; S3:S2
	ret

Display_2
	mov #Display_3,R4

	mov.b #081h,&LCDM2	; S3:S2
	ret

Display_3
	mov #Display_4,R4

	mov.b #082h,&LCDM2	; S1:S0	
	ret

Display_4
	mov #Display_5,R4

	mov.b #082h,&LCDM2	; S1:S0	
	ret

Display_5
	mov #Display_6,R4

	mov.b #082h,&LCDM2	; S1:S0	
	ret

Display_6
	mov #Display_7,R4

	mov.b #084h,&LCDM2	; S3:S2
	ret

Display_7
	mov #Display_8,R4

	mov.b #084h,&LCDM2	; S3:S2
	ret

Display_8
	mov #Display_9,R4

	mov.b #084h,&LCDM2	; S3:S2
	ret

Display_9
	mov #Display_10,R4

	mov.b #088h,&LCDM2	; S3:S2
	ret

Display_10
	mov #Display_11,R4

	mov.b #088h,&LCDM2	; S3:S2
	ret

Display_11
	mov #Display_0,R4

	mov.b #088h,&LCDM2	; S3:S2
	ret

Init_UART
	; Assumes bis.b #003h,&P4SEL elsewhere
	; Setup Tx pin direction
	bis.b #BIT0,&P4DIR
	; Rx pin is input

	; Enable module
	bis.b #(UTXE1|URXE1),&ME2

	; Setup 2400 baud with ACLK
	mov.b #00Dh,&U1BR0
	mov.b #000h,&U1BR1
	mov.b #06Bh,&U1MCTL
	
	; Reset UART
	mov.b #(CHAR),&U1CTL
	mov.b #(TXEPT+010h),&U1TCTL
	mov.b #(URXEIE),&U1RCTL

	; Set Tx Flag
	bis.b #(UTXIFG1),&IFG2
		
	; Enable receive interrupts
	bis.b #(URXIE1),&IE2

	ret

Int_BT
	; Wake processor
	bic #(CPUOFF+OSCOFF+SCG1+SCG0),0(SP)
	reti
	
Int_URX1
; Handle received byte
	mov.b &U1RXBUF,R15
	inc R15
	mov.b R15,&U1TXBUF

	reti	

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Interrupt vectors
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;ORG 0FFE0h
  .pseg	int_vect,abs=0FFE0h
  .data  Int_BT			  ; Basic Timer
  .pseg reset_vect,abs=0FFE2h
  .data	RESET
  .pseg reset_vect1,abs=0FFE4h
  .data	RESET
  .pseg reset_vect2,abs=0FFE6h
  .data	Int_URX1
  .pseg reset_vect3,abs=0FFE8h
  .data	RESET
  .pseg reset_vect4,abs=0FFEAh
  .data	RESET
  .pseg reset_vect5,abs=0FFECh
  .data	RESET
  .pseg reset_vect6,abs=0FFEEh
  .data	RESET
  .pseg reset_vect7,abs=0FFF0h
  .data	RESET
  .pseg reset_vect8,abs=0FFF2h
  .data	RESET
  .pseg reset_vect9,abs=0FFF4h
  .data	RESET
  .pseg reset_vecta,abs=0FFF6h
  .data	RESET
  .pseg reset_vectb,abs=0FFF8h
  .data	RESET
  .pseg reset_vectc,abs=0FFFAh
  .data	RESET
  .pseg reset_vectd,abs=0FFFCh
  .data	RESET
  .pseg reset_vecte,abs=0FFFEh
  .data	RESET
  END